# -*- coding: utf-8 -*-
from DateTime import DateTime

import Missing
import datetime
import json


def custom_json_handler(obj):
    if obj == Missing.Value:
        return None
    if type(obj) in (datetime.datetime, datetime.date):
        return obj.isoformat()
    if type(obj) == DateTime:
        dt = DateTime(obj)
        return dt.ISO()
    if type(obj) == set:
        return list(obj)
    return obj


def json_dumps(data):
    return json.dumps(data, default=custom_json_handler)


# can eventually provide custom handling here if we want
json_loads = json.loads



# -*- coding: utf-8 -*-
from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.interfaces import IStructureAction
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component.hooks import getSite
from zope.i18n import translate
from zope.interface import implementer


@implementer(IStructureAction)
class TagsAction(object):

    template = ViewPageTemplateFile('templates/tags.pt')
    order = 6

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_options(self):
        base_vocabulary = '%s/@@getVocabulary?name=' % getSite().absolute_url()
        return {
            'title': translate(_('Tags'), context=self.request),
            'id': 'tags',
            'icon': 'tags',
            'url': self.context.absolute_url() + '/@@fc-tags',
            'form': {
                'template': self.template(
                    vocabulary_url='%splone.app.vocabularies.Keywords' % (
                        base_vocabulary)
                )
            }
        }


class TagsActionView(ContentsBaseAction):
    required_obj_permission = 'Modify portal content'
    success_msg = _('Successfully updated tags on items')
    failure_msg = _('Failed to modify tags on items')

    def action(self, obj):
        toadd = self.request.form.get('toadd')
        if toadd:
            toadd = set(toadd.split(','))
        else:
            toadd = set([])
        toremove = self.request.get('toremove')
        if toremove:
            toremove = set(toremove.split(','))
        else:
            toremove = set([])
        tags = set(obj.Subject())
        tags = tags - toremove
        tags = tags | toadd
        obj.setSubject(list(tags))
        obj.reindexObject()



# -*- coding: utf-8 -*-


def reindexOnModify(content, event):
    """When an object is modified, re-index it in the catalog
    """
    if event.object is not content:
        return
    content.reindexObject(idxs=getattr(event, 'descriptions', []))



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from Products.CMFCore.utils import getToolByName
from zope.configuration import xmlconfig
from zope.interface import implementer
from zope.interface import provider
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


@implementer(IVocabularyFactory)
class ExampleVocabulary(object):

    def __call__(self, context, query=None):
        items = [u'One', u'Two', u'Three']
        tmp = SimpleVocabulary([
            SimpleTerm(it.lower(), it.lower(), it)
            for it in items
            if query is None
            or query.lower() in it.lower()
        ])
        tmp.test = 1
        return tmp


@provider(IVocabularyFactory)
def ExampleFunctionVocabulary(context, query=None):
    items = [u'First', u'Second', u'Third']
    tmp = SimpleVocabulary([
        SimpleTerm(it.lower(), it.lower(), it)
        for it in items
        if query is None
        or query.lower() in it.lower()
    ])
    return tmp


class PloneAppContent(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, )

    USER_NAME = 'johndoe'
    USER_PASSWORD = 'secret'
    MEMBER_NAME = 'janedoe'
    MEMBER_PASSWORD = 'secret'
    USER_WITH_FULLNAME_NAME = 'jim'
    USER_WITH_FULLNAME_FULLNAME = 'Jim Fulton'
    USER_WITH_FULLNAME_PASSWORD = 'secret'
    MANAGER_USER_NAME = 'manager'
    MANAGER_USER_PASSWORD = 'secret'

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import plone.app.content
        xmlconfig.file(
            'configure.zcml',
            plone.app.content,
            context=configurationContext
        )

    def setUpPloneSite(self, portal):
        # Creates some users
        acl_users = getToolByName(portal, 'acl_users')
        acl_users.userFolderAddUser(
            self.USER_NAME,
            self.USER_PASSWORD,
            [],
            [],
        )
        acl_users.userFolderAddUser(
            self.MEMBER_NAME,
            self.MEMBER_PASSWORD,
            ['Member'],
            [],
        )
        acl_users.userFolderAddUser(
            self.USER_WITH_FULLNAME_NAME,
            self.USER_WITH_FULLNAME_PASSWORD,
            ['Member'],
            [],
        )
        mtool = getToolByName(portal, 'portal_membership', None)
        mtool.addMember('jim', 'Jim', ['Member'], [])
        mtool.getMemberById('jim').setMemberProperties(
            {"fullname": 'Jim Fult\xc3\xb8rn'})

        acl_users.userFolderAddUser(
            self.MANAGER_USER_NAME,
            self.MANAGER_USER_PASSWORD,
            ['Manager'],
            [],
        )
        portal.portal_workflow.setDefaultChain("simple_publication_workflow")


class PloneAppContentAT(PloneAppContent):

    def setUpZope(self, app, configurationContext):
        super(PloneAppContentAT, self).setUpZope(app, configurationContext)
        import Products.ATContentTypes
        xmlconfig.file('configure.zcml',
                       Products.ATContentTypes,
                       context=configurationContext)
        z2.installProduct(app, 'Products.ATContentTypes')

    def setUpPloneSite(self, portal):
        super(PloneAppContentAT, self).setUpPloneSite(portal)
        self.applyProfile(portal, 'Products.ATContentTypes:default')


PLONE_APP_CONTENT_FIXTURE = PloneAppContent()
PLONE_APP_CONTENT_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENT_FIXTURE, ),
    name="PloneAppContent:Integration")
PLONE_APP_CONTENT_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENT_FIXTURE, ),
    name="PloneAppContent:Functional")


# Dexterity test layers
PLONE_APP_CONTENT_DX_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE, ),
    name="PloneAppContentDX:Integration")
PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE, ),
    name="PloneAppContentDX:Functional")


# AT test layers
PLONE_APP_CONTENT_AT_FIXTURE = PloneAppContentAT()
PLONE_APP_CONTENT_AT_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENT_AT_FIXTURE, ),
    name="PloneAppContentAT:Integration")
PLONE_APP_CONTENT_AT_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENT_AT_FIXTURE, ),
    name="PloneAppContentAT:Functional")



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import field
from z3c.form import form
from zope.interface import Interface
from zope.publisher.browser import BrowserView
from zope.schema import Datetime
from zope.schema.fieldproperty import FieldProperty


class IContentStatusHistoryDates(Interface):
    """ Interface for the two dates on content status history view
    """

    effective_date = Datetime(
        title=_(u"label_effective_date",
                default=u"Publishing Date"),
        description=_(
            u"help_effective_date",
            default=u"The date when the item will be published. If no "
                    u"date is selected the item will be published immediately."
        ),
        required=False)

    expiration_date = Datetime(
        title=_(u"label_expiration_date",
                default=u"Expiration Date"),
        description=_(
            u"help_expiration_date",
            default=u"The date when the item expires. This will automatically "
                    u"make the item invisible for others at the given date. "
                    u"If no date is chosen, it will never expire."
            ),
        required=False)


class ContentStatusHistoryDatesForm(form.Form):
    fields = field.Fields(IContentStatusHistoryDates)
    ignoreContext = True
    label = "Content status history dates"

    effective_date = FieldProperty(
        IContentStatusHistoryDates['effective_date']
    )
    expiration_date = FieldProperty(
        IContentStatusHistoryDates['expiration_date']
    )


class ContentStatusHistoryView(BrowserView):

    template = ViewPageTemplateFile('templates/content_status_history.pt')

    def __init__(self, context, request):
        super(ContentStatusHistoryView, self).__init__(context, request)

        self.dates_form = ContentStatusHistoryDatesForm(context, request)
        self.dates_form.updateWidgets()
        self.plone_utils = getToolByName(context, 'plone_utils')
        self.errors = {}

    def __call__(self, workflow_action=None, paths=[], comment="",
                 effective_date=None, expiration_date=None,
                 include_children=False, *args):

        data = self.dates_form.extractData()
        if self.request.get('form.widgets.effective_date-calendar', None) \
           and data:
            effective_date = data[0]['effective_date'].strftime(
                "%Y-%m-%d %H:%M"
            )

        if self.request.get('form.widgets.expiration_date-calendar', None) \
           and data:
            expiration_date = data[0]['expiration_date'].strftime(
                "%Y-%m-%d %H:%M"
            )

        if self.request.get('form.button.Cancel', None):
            return self.request.RESPONSE.redirect(
                "%s/view" % self.context.absolute_url())

        if self.request.get('form.submitted', None):
            self.validate(workflow_action=workflow_action, paths=paths)
            if self.errors:
                self.plone_utils.addPortalMessage(
                    _(u'Please correct the indicated errors.'), 'error')
                return self.template()

        if self.request.get('form.button.Publish', None):
            return self.context.restrictedTraverse('content_status_modify')(
                workflow_action=workflow_action,
                comment=comment,
                effective_date=effective_date,
                expiration_date=expiration_date)

        if self.request.get('form.button.FolderPublish', None):
            self.context.restrictedTraverse('folder_publish')(
                workflow_action=workflow_action,
                paths=paths,
                comment=comment,
                expiration_date=expiration_date,
                effective_date=effective_date,
                include_children=include_children)

        return self.template()

    def validate(self, workflow_action=None, paths=[]):
        if workflow_action is None:
            self.errors['workflow_action'] = _(
                u'You must select a publishing action.')

        if not paths:
            self.errors['paths'] = _(
                u'You must select content to change.')
            # If there are no paths, it's mostly a mistake
            # Set paths using orgi_paths, otherwise users are getting confused
            orig_paths = self.request.get('orig_paths')
            self.request.set('paths', orig_paths)



# -*- coding: utf-8 -*-
from plone.app.content.testing import PLONE_APP_CONTENT_AT_FUNCTIONAL_TESTING
from plone.app.content.testing import PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.testing.z2 import Browser
import transaction
import unittest

FOLDER = {'id': 'testfolder',
          'title': 'Test Folder',
          'description': 'Test Folder Description'}

DOCUMENT = {'id': 'testdoc',
            'title': 'Test Document',
            'description': 'Test Document Description'}


class SelectDefaultPageDXTestCase(unittest.TestCase):

    layer = PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.portal.acl_users.userFolderAddUser(
            'editor', 'secret', ['Editor'], [])

        self._create_structure()
        transaction.commit()

        self.browser = Browser(self.layer['app'])
        self.browser.addHeader('Authorization',
                               'Basic %s:%s' % ('editor', 'secret'))

    def tearDown(self):
        self.portal.manage_delObjects(ids=FOLDER['id'])
        transaction.commit()

    def _createFolder(self):
        self.portal.invokeFactory(id=FOLDER['id'], type_name='Folder')
        folder = getattr(self.portal, FOLDER['id'])
        folder.setTitle(FOLDER['title'])
        folder.setDescription(FOLDER['description'])
        folder.reindexObject()
        # we don't want it in the navigation
        # folder.setExcludeFromNav(True)
        return folder

    def _createDocument(self, context):
        context.invokeFactory(id=DOCUMENT['id'], type_name='Document')
        doc = getattr(context, DOCUMENT['id'])
        doc.setTitle(DOCUMENT['title'])
        doc.setDescription(DOCUMENT['description'])
        doc.reindexObject()
        # we don't want it in the navigation
        # doc.setExcludeFromNav(True)
        return doc

    def _create_structure(self):
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        folder = self._createFolder()
        self._createDocument(folder)
        return folder

    def test_select_default_page_view(self):
        """Check that the form can be rendered."""
        folder = self.portal.testfolder

        self.browser.open('%s/@@select_default_page' % folder.absolute_url())

        self.assertTrue('Select default page' in self.browser.contents)
        self.assertTrue('id="testdoc"' in self.browser.contents)

    def test_select_default_page_view_with_folderish_type(self):
        """Check if folderish types are available."""
        folder = self.portal.testfolder
        folder.invokeFactory(id=FOLDER['id'], type_name='Folder')
        folder2 = getattr(folder, FOLDER['id'])
        folder.setTitle(FOLDER['title'])
        folder2.reindexObject()
        folder_fti = self.portal.portal_types['Folder']
        folder_fti.manage_changeProperties(
            filter_content_types=True, allowed_content_types=[])
        view = folder.restrictedTraverse('@@select_default_page')()

        self.assertTrue('id="testdoc"' in view)
        self.assertTrue('id="testfolder"' in view)

    def test_default_page_action_cancel(self):
        """Check the Cancel action."""
        folder = self.portal.testfolder

        self.browser.open('%s/@@select_default_page' % folder.absolute_url())
        cancel_button = self.browser.getControl(name='form.buttons.Cancel')
        cancel_button.click()

        self.assertEqual(self.browser.url, folder.absolute_url())
        self.assertIs(folder.getDefaultPage(), None)

    def test_default_page_action_save(self):
        """Check the Save action."""
        folder = self.portal.testfolder
        self.browser.open('%s/@@select_default_page' % folder.absolute_url())

        submit_button = self.browser.getControl(name='form.buttons.Save')
        submit_button.click()

        self.assertEqual(self.browser.url, folder.absolute_url())
        self.assertEqual(folder.getDefaultPage(), 'testdoc')


class SelectDefaultPageATTestCase(SelectDefaultPageDXTestCase):

    layer = PLONE_APP_CONTENT_AT_FUNCTIONAL_TESTING



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from zope.component import getUtility
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from plone.registry.interfaces import IRegistry
from zope.component import getMultiAdapter


class DefaultViewSelectionView(BrowserView):

    def isValidTemplate(self, templateId):
        return templateId in [a[0] for a in self.vocab]

    def canSelectDefaultPage(self):
        if not self.context.isPrincipiaFolderish:
            return False
        return self.context.canSetDefaultPage()

    @property
    def vocab(self):
        return self.context.getAvailableLayouts()

    @property
    def selectedLayout(self):
        if not self.context_state.is_default_page():
            return self.context.getLayout()
        else:
            return ''

    def selectViewTemplate(self):
        templateId = self.request.get('templateId')

        if self.isValidTemplate(templateId):
            self.context.setLayout(templateId)

        self.request.response.redirect(self.context.absolute_url() + '/view')

    @property
    def action_url(self):
        return '{0:s}/select_default_view'.format(
            self.context_state.object_url())

    def __call__(self):

        self.context_state = getMultiAdapter(
            (self.context, self.request), name='plone_context_state')

        templateId = self.request.form.get('templateId', False)
        if templateId:
            plone_utils = getToolByName(self.context, 'plone_utils')
            # Make sure this is a valid template
            if self.isValidTemplate(templateId):
                # Update the template
                self.context.setLayout(templateId)
                plone_utils.addPortalMessage(u'View changed.')
            else:
                plone_utils.addPortalMessage(u'Invalid view.', type="error")
                return self.index()

        if templateId or self.request.form.get('form.buttons.Cancel', False):
            # Redirect to view
            self.request.response.redirect(
                '%s/view' % self.context_state.object_url())

        return self.index()


class DefaultPageSelectionView(BrowserView):

    def __call__(self):
        if 'form.buttons.Save' in self.request.form:
            if 'objectId' not in self.request.form:
                message = _(u'Please select an item to use.')
                msgtype = 'error'
            else:
                objectId = self.request.form['objectId']

                if objectId not in self.context.objectIds():
                    message = _(u'There is no object with short name ${name} '
                                u'in this folder.',
                                mapping={u'name': objectId})
                    msgtype = 'error'
                else:
                    self.context.setDefaultPage(objectId)
                    message = _(u'View changed.')
                    msgtype = 'info'
                    self.request.response.redirect(self.context.absolute_url())
            IStatusMessage(self.request).add(message, msgtype)
        elif 'form.buttons.Cancel' in self.request.form:
            self.request.response.redirect(self.context.absolute_url())

        return self.index()

    def get_selectable_items(self):
        """ Return brains in this container that can be used as default_pages
        """
        context = aq_inner(self.context)
        registry = getUtility(IRegistry)
        view_types = registry.get(
            'plone.types_use_view_action_in_listings', [])
        default_page_types = registry.get('plone.default_page_types', [])
        portal_types = getToolByName(self.context, 'portal_types')

        results = []
        for brain in context.getFolderContents():
            portal_type = brain.portal_type
            if portal_type in view_types:
                # Skip files and images
                continue

            if portal_type in default_page_types:
                # Allow types that are explicitly in default_page_types
                results.append(brain)
                continue

            if brain.is_folderish:
                fti = portal_types.get(portal_type)
                if not fti:
                    continue
                if fti.filter_content_types and fti.allowed_content_types or \
                        not fti.filter_content_types:
                    # Disallow folderish types if you can't add any content.
                    # To override you have to add type to default_page_types
                    continue
            results.append(brain)
        return results



# -*- coding: utf-8 -*-
from AccessControl import Unauthorized
from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.interfaces import IStructureAction
from Products.CMFPlone import PloneMessageFactory as _
from ZODB.POSException import ConflictError
from zope.i18n import translate
from zope.interface import implementer


@implementer(IStructureAction)
class PasteAction(object):

    order = 3

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_options(self):
        return {
            'title': translate(_('Paste'), context=self.request),
            'id': 'paste',
            'icon': 'paste',
            'url': self.context.absolute_url() + '/@@fc-paste'
        }


class PasteActionView(ContentsBaseAction):
    required_obj_permission = 'Copy or Move'
    success_msg = _('Successfully pasted items')
    failure_msg = _('Failed to paste items')

    def __call__(self):
        self.protect()
        self.errors = []

        self.dest = self.site.restrictedTraverse(
            str(self.request.form['folder'].lstrip('/')))

        try:
            self.dest.manage_pasteObjects(self.request['__cp'])
        except ConflictError:
            raise
        except Unauthorized:
            # avoid this unfriendly exception text:
            # "You are not allowed to access 'manage_pasteObjects' in this
            # context"
            self.errors.append(
                _(u'You are not authorized to paste ${title} here.',
                    mapping={u'title': self.objectTitle(self.dest)}))
        except ValueError as e:
            if 'Disallowed subobject type: ' in e.message:
                msg_parts = e.message.split(':')
                self.errors.append(
                    _(u'Disallowed subobject type "${type}"',
                        mapping={u'type': msg_parts[1].strip()}))
            else:
                raise e

        return self.message()



# -*- coding: utf-8 -*-
from Products.Five import BrowserView
from plone.app.querystring.interfaces import IQuerystringRegistryReader
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
import json


class QueryStringIndexOptions(BrowserView):

    def __call__(self):
        registry = getUtility(IRegistry)
        config = IQuerystringRegistryReader(registry)()
        self.request.response.setHeader(
            'Content-Type', 'application/json; charset=utf-8'
        )
        return json.dumps(config)



# -*- coding: utf-8 -*-
from Acquisition import Implicit
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.adding import ContentAdding


class CMFAdding(Implicit, ContentAdding):
    """An adding view with a less silly next-url
    """

    # We need to do this to get proper traversal URLs - otherwise, the
    # <base /> tag is messed up.
    id = '+'

    def add(self, content):
        content = super(CMFAdding, self).add(content)
        # We need to ensure that we finish type construction, not at least
        # to set the correct permissions based on the workflow
        getToolByName(content, 'portal_types')

        return content

    def nextURL(self):
        return "%s/%s/view" % (self.context.absolute_url(), self.contentName)



# -*- coding: utf-8 -*-
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.PortalFolder import PortalFolderBase
from Products.CMFPlone.DublinCore import DefaultDublinCoreImpl
from plone.app.content.interfaces import IReindexOnModify
from zope.container.contained import Contained
from zope.container.interfaces import IContainer
from zope.interface import implementer


@implementer(IContainer)
class OFSContainer(object):
    """A folder that's also a container.

    Borrowed in part from megrok.five.
    """

    isPrincipiaFolderish = 1

    def __init__(self, id=None):
        if id is not None:
            self.id = id
    # fulfill IContainer interface

    def keys(self):
        return self.objectIds()

    def values(self):
        return self.objectValues()

    def items(self):
        return self.objectItems()

    def get(self, name, default=None):
        return getattr(self, name, default)
    # __getitem__ is already implemented by ObjectManager

    def __setitem__(self, name, obj):
        name = name.encode('ascii')  # may raise if there's a bugus id
        self._setObject(name, obj)

    def __delitem__(self, name):
        self.manage_delObjects([name])

    def __contains__(self, name):
        return self.hasObject(name)

    def __iter__(self):
        return iter(self.objectIds())

    def __len__(self):
        return len(self.objectIds())

# Notes on this insane mixing of classes:
#
#  - OFSContainer gives us Zope3-like container operations, and we want that
#       to take priority so it comes first
#  - CMFCatalogAware gives us catalog functionality. So does PortalContent,
#       but PortalFolderBase overrides indexObject() and friends to do
#       nothing.
#  - PortalFolderBase gives folder-like behaviour. It needs to come before
#       PortalContent, otherwise objectIds() and friends don't work
#  - PortalContent gives us SearchableText and WebDAV
#  - DublinCoreImpl gives us DC fields
#  - Contained gives us Zope3-like containment
#
# ... I WANT AN ADAPTER!


@implementer(IReindexOnModify)
class Container(OFSContainer, CMFCatalogAware, PortalFolderBase, PortalContent,
                DefaultDublinCoreImpl, Contained):
    """A base class mixing in CMFish, contentish, containerish, containedish,
    dublincoreish behaviour.
    """

    def __init__(self, id=None, **kwargs):
        OFSContainer.__init__(self, id, **kwargs)
        PortalFolderBase.__init__(self, id, **kwargs)
        DefaultDublinCoreImpl.__init__(self, **kwargs)

        if id is not None:
            self.id = id



# -*- coding: utf-8 -*-
from plone.app.content.testing import PLONE_APP_CONTENT_DX_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.dexterity.fti import DexterityFTI

import json
import mock
import unittest


class ContentsPasteTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        # TYPE 1
        type1_fti = DexterityFTI('type1')
        type1_fti.klass = 'plone.dexterity.content.Container'
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = ['type1']
        type1_fti.behaviors = (
            'Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes',  # noqa
            'plone.app.dexterity.behaviors.metadata.IBasic'
        )
        self.portal.portal_types._setObject('type1', type1_fti)
        self.type1_fti = type1_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.invokeFactory('type1', id='it1', title='Item 1')

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_paste_success(self):
        """Copy content item and paste in portal root.
        """
        # # setup copying via @@fc-copy
        # from plone.uuid.interfaces import IUUID
        # self.request['selection'] = [IUUID(self.portal.it1)]
        # self.portal.restrictedTraverse('@@fc-copy')()

        self.request['__cp'] = self.portal.manage_copyObjects(['it1'])
        self.request.form['folder'] = '/'
        res = self.portal.restrictedTraverse('@@fc-paste')()

        res = json.loads(res)
        self.assertEqual(res['status'], 'success')
        self.assertEqual(len(self.portal.contentIds()), 2)

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_paste_success_paste_in_itself(self):
        """Copy content item and paste in itself. Because we can.
        """
        self.request['__cp'] = self.portal.manage_copyObjects(['it1'])
        self.request.form['folder'] = '/it1'
        res = self.portal.it1.restrictedTraverse('@@fc-paste')()

        res = json.loads(res)
        self.assertEqual(res['status'], 'success')
        self.assertEqual(len(self.portal.it1.contentIds()), 1)

    @mock.patch('plone.app.content.browser.contents.ContentsBaseAction.protect', lambda x: True)  # noqa
    def test_paste_fail_constraint(self):
        """Fail pasting content item in itself when folder constraints don't
        allow to.
        """
        self.type1_fti.allowed_content_types = []  # set folder constraints
        self.request['__cp'] = self.portal.manage_copyObjects(['it1'])
        self.request.form['folder'] = '/it1'
        res = self.portal.it1.restrictedTraverse('@@fc-paste')()

        res = json.loads(res)
        self.assertEqual(res['status'], 'warning')
        self.assertEqual(len(self.portal.it1.contentIds()), 0)


class AllowUploadViewTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        # TYPE 1
        type1_fti = DexterityFTI('type1')
        type1_fti.klass = 'plone.dexterity.content.Container'
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = []
        type1_fti.behaviors = (
            'plone.app.dexterity.behaviors.metadata.IBasic'
        )
        self.portal.portal_types._setObject('type1', type1_fti)
        self.type1_fti = type1_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.invokeFactory('type1', id='it1', title='Item 1')

    def test_allow_upload(self):
        """Test, if file or images are allowed in a container in different FTI
        configurations.
        """

        # Test none allowed
        self.type1_fti.allowed_content_types = []
        allow_upload = self.portal.it1.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload['allowUpload'], False)
        self.assertEqual(allow_upload['allowImages'], False)
        self.assertEqual(allow_upload['allowFiles'], False)

        # Test images allowed
        self.type1_fti.allowed_content_types = ['Image']
        allow_upload = self.portal.it1.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload['allowUpload'], True)
        self.assertEqual(allow_upload['allowImages'], True)
        self.assertEqual(allow_upload['allowFiles'], False)

        # Test files allowed
        self.type1_fti.allowed_content_types = ['File']
        allow_upload = self.portal.it1.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload['allowUpload'], True)
        self.assertEqual(allow_upload['allowImages'], False)
        self.assertEqual(allow_upload['allowFiles'], True)

        # Test images and files allowed
        self.type1_fti.allowed_content_types = ['Image', 'File']
        allow_upload = self.portal.it1.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload['allowUpload'], True)
        self.assertEqual(allow_upload['allowImages'], True)
        self.assertEqual(allow_upload['allowFiles'], True)

        # Test files allowed, path via request variable
        self.type1_fti.allowed_content_types = ['File']
        # First, test on Portal root to see the difference
        allow_upload = self.portal.restrictedTraverse('@@allow_upload')
        allow_upload = json.loads(allow_upload())
        self.assertEqual(allow_upload['allowUpload'], True)
        self.assertEqual(allow_upload['allowImages'], True)
        self.assertEqual(allow_upload['allowFiles'], True)
        # Then, with path set to sub item
        allow_upload = self.portal.restrictedTraverse('@@allow_upload')
        allow_upload.request.form['path'] = '/plone/it1'
        allow_upload = json.loads(allow_upload())
        self.assertEqual(allow_upload['allowUpload'], True)
        self.assertEqual(allow_upload['allowImages'], False)
        self.assertEqual(allow_upload['allowFiles'], True)



# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.interfaces import IStructureAction
from plone.dexterity.interfaces import IDexterityContent
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component.hooks import getSite
from zope.i18n import translate
from zope.interface import implementer


@implementer(IStructureAction)
class PropertiesAction(object):

    template = ViewPageTemplateFile('templates/properties.pt')
    order = 8

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_options(self):
        base_vocabulary = '%s/@@getVocabulary?name=' % getSite().absolute_url()
        return {
            'title': translate(_('Properties'), context=self.request),
            'id': 'properties',
            'icon': 'edit',
            'url': self.context.absolute_url() + '/@@fc-properties',
            'form': {
                'title': _('Modify properties on items'),
                'template': self.template(
                    vocabulary_url='%splone.app.vocabularies.Users' % (
                        base_vocabulary)
                )
            }
        }


class PropertiesActionView(ContentsBaseAction):
    success_msg = _(u'Successfully updated metadata')
    failure_msg = _(u'Failure updating metadata')
    required_obj_permission = 'Modify portal content'

    def __call__(self):
        self.effectiveDate = self.request.form.get('effectiveDate')
        self.expirationDate = self.request.form.get('expirationDate')
        self.copyright = self.request.form.get('copyright')
        self.contributors = self.request.form.get('contributors')
        if self.contributors:
            self.contributors = self.contributors.split(',')
        else:
            self.contributors = []
        self.creators = self.request.form.get('creators', '')
        if self.creators:
            self.creators = self.creators.split(',')
        self.exclude = self.request.form.get('exclude-from-nav')
        return super(PropertiesActionView, self).__call__()

    def dx_action(self, obj):
        if self.effectiveDate and hasattr(obj, 'effective_date'):
            obj.effective_date = DateTime(self.effectiveDate)
        if self.expirationDate and hasattr(obj, 'expiration_date'):
            obj.expiration_date = DateTime(self.expirationDate)
        if self.copyright and hasattr(obj, 'rights'):
            obj.rights = self.copyright
        if self.contributors and hasattr(obj, 'contributors'):
            obj.contributors = tuple(self.contributors)
        if self.creators and hasattr(obj, 'creators'):
            obj.creators = tuple(self.creators)
        if self.exclude and hasattr(obj, 'exclude_from_nav'):
            obj.exclude_from_nav = self.exclude == 'yes'

    def action(self, obj):
        if IDexterityContent.providedBy(obj):
            self.dx_action(obj)
        else:
            if self.effectiveDate:
                try:
                    obj.setEffectiveDate(DateTime(self.effectiveDate))
                except AttributeError:
                    pass
            if self.expirationDate:
                try:
                    obj.setExpirationDate(DateTime(self.expirationDate))
                except AttributeError:
                    pass
            if self.copyright:
                try:
                    obj.setRights(self.copyright)
                except AttributeError:
                    pass
            if self.contributors:
                try:
                    obj.setContributors(self.contributors)
                except AttributeError:
                    pass
            if self.creators:
                try:
                    obj.setCreators(self.creators)
                except AttributeError:
                    pass
            if self.exclude:
                try:
                    obj.setExcludeFromNav(self.exclude == 'yes')
                except AttributeError:
                    pass
        obj.reindexObject()



# -*- coding: utf-8 -*-
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ZTUtils import make_query
from plone.batching import Batch
from plone.batching.browser import BatchView
from plone.memoize import instance
from zope.i18nmessageid import MessageFactory
from zope.publisher.browser import BrowserView
import urllib

_ = MessageFactory('plone')


class TableBatchView(BatchView):

    def make_link(self, pagenumber):
        batchlinkparams = self.request.form.copy()
        return '%s?%s' % (self.request.ACTUAL_URL,
                          make_query(batchlinkparams,
                                     {'pagenumber': pagenumber}))


class Table(object):
    """
    The table renders a table with sortable columns etc.

    It is meant to be subclassed to provide methods for getting specific table
    info.
    """

    def __init__(self, request, base_url, view_url, items,
                 show_sort_column=False, buttons=None, pagesize=20,
                 show_select_column=True, show_size_column=True,
                 show_modified_column=True, show_status_column=True):
        if buttons is None:
            buttons = []
        self.request = request
        self.context = None  # Need for view pagetemplate

        self.base_url = base_url
        self.view_url = view_url
        self.url = view_url
        self.items = items
        self.show_sort_column = show_sort_column
        self.show_select_column = show_select_column
        self.show_size_column = show_size_column
        self.show_modified_column = show_modified_column
        self.show_status_column = show_status_column
        self.buttons = buttons
        self.default_page_size = 20
        self.pagesize = pagesize
        self.show_all = request.get('show_all', '').lower() == 'true'

        selection = request.get('select')
        if selection == 'screen':
            self.selectcurrentbatch = True
        elif selection == 'all':
            self.selectall = True

        self.pagenumber = int(request.get('pagenumber', 1))

    def msg_select_item(self, item):
        title_or_id = (item.get('title_or_id') or item.get('title') or
                       item.get('Title') or item.get('id') or item['getId'])
        return _(u'checkbox_select_item',
                 default=u"Select ${title}",
                 mapping={'title': safe_unicode(title_or_id)})

    @property
    def within_batch_size(self):
        return len(self.items) <= self.pagesize

    def set_checked(self, item):
        selected = self.selected(item)
        item['checked'] = selected and 'checked' or None
        item['table_row_class'] = item.get('table_row_class', '')
        if selected:
            item['table_row_class'] += ' selected'

    @property
    @instance.memoize
    def batch(self):
        pagesize = self.pagesize
        if self.show_all:
            pagesize = len(self.items)
        b = Batch.fromPagenumber(self.items,
                                 pagesize=pagesize,
                                 pagenumber=self.pagenumber)
        map(self.set_checked, b)
        return b

    render = ViewPageTemplateFile("table.pt")

    def batching(self):
        return TableBatchView(self.context, self.request)(self.batch)

    # options
    _selectcurrentbatch = False
    _select_all = False

    def _get_select_currentbatch(self):
        return self._selectcurrentbatch

    def _set_select_currentbatch(self, value):
        self._selectcurrentbatch = value
        if self._selectcurrentbatch and self.show_all or (
                len(self.items) <= self.pagesize):
            self.selectall = True

    selectcurrentbatch = property(_get_select_currentbatch,
                                  _set_select_currentbatch)

    def _get_select_all(self):
        return self._select_all

    def _set_select_all(self, value):
        self._select_all = bool(value)
        if self._select_all:
            self._selectcurrentbatch = True

    selectall = property(_get_select_all, _set_select_all)

    @property
    def show_select_all_items(self):
        return self.selectcurrentbatch and not self.selectall

    def get_nosort_class(self):
        """
        """
        return "nosort"

    @property
    def selectall_url(self):
        return self.selectnone_url + '&select=all'

    @property
    def selectscreen_url(self):
        return self.selectnone_url + '&select=screen'

    @property
    def selectnone_url(self):
        base = self.view_url + '?pagenumber=%s&pagesize=%s' % (
            self.pagenumber, self.pagesize)
        if self.show_all:
            base += '&show_all=true'
        return base

    @property
    def show_all_url(self):
        return self.view_url + '?show_all=true'

    def selected(self, item):
        if self.selectcurrentbatch:
            return True
        return False

    @property
    def viewname(self):
        return self.view_url.split('?')[0].split('/')[-1]

    def quote_plus(self, string):
        return urllib.quote_plus(string)


class TableBrowserView(BrowserView):
    '''Base class which can be used from a AJAX view

    Subclasses only need to set the table property to a different
    class.'''

    table = None

    def update_table(self, pagenumber='1', sort_on='getObjPositionInParent',
                     show_all=False):
        self.request.set('sort_on', sort_on)
        self.request.set('pagenumber', pagenumber)
        table = self.table(self.context, self.request,
                           contentFilter={'sort_on': sort_on})
        return table.render()



# -*- coding: utf-8 -*-
from AccessControl import Unauthorized
from AccessControl.Permissions import delete_objects
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.interfaces import IStructureAction
from zope.component import getMultiAdapter
from zope.component.hooks import getSite
from zope.i18n import translate
from zope.interface import implementer
import json


@implementer(IStructureAction)
class DeleteAction(object):

    template = ViewPageTemplateFile('templates/delete.pt')
    order = 4

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_options(self):
        return {
            'title': translate(_('Delete'), context=self.request),
            'id': 'delete',
            'icon': 'trash',
            'context': 'danger',
            'url': self.context.absolute_url() + '/@@fc-delete',
            'form': {
                'title': _('Delete selected items'),
                'submitText': _('Yes'),
                'submitContext': 'danger',
                'template': self.template(),
                'closeText': _('No'),
                'dataUrl': self.context.absolute_url() + '/@@fc-delete'
            }
        }


class DeleteActionView(ContentsBaseAction):
    required_obj_permission = delete_objects
    success_msg = _('Successfully delete items')
    failure_msg = _('Failed to delete items')

    def __call__(self):
        if self.request.form.get('render') == 'yes':
            confirm_view = getMultiAdapter((getSite(), self.request),
                                           name='delete_confirmation_info')
            selection = self.get_selection()
            catalog = getToolByName(self.context, 'portal_catalog')
            brains = catalog(UID=selection)
            items = [i.getObject() for i in brains]
            self.request.response.setHeader(
                'Content-Type', 'application/json; charset=utf-8'
            )
            return json.dumps({
                'html': confirm_view(items)
            })
        else:
            return super(DeleteActionView, self).__call__()

    def action(self, obj):
        parent = obj.aq_inner.aq_parent
        title = self.objectTitle(obj)

        try:
            lock_info = obj.restrictedTraverse('@@plone_lock_info')
        except AttributeError:
            lock_info = None

        if lock_info is not None and lock_info.is_locked():
            self.errors.append(_(u'${title} is locked and cannot be deleted.',
                                 mapping={u'title': title}))
            return
        else:
            try:
                parent.manage_delObjects(obj.getId(), self.request)
            except Unauthorized:
                self.errors.append(
                    _(u'You are not authorized to delete ${title}.',
                        mapping={u'title': self.objectTitle(self.dest)}))



# -*- coding: utf-8 -*-
from Testing import ZopeTestCase as ztc
from plone.app.content.tests.base import ContentFunctionalTestCase
import doctest
import unittest


def test_suite():
    return unittest.TestSuite((
        ztc.ZopeDocFileSuite(
            'basecontent.rst',
            package='plone.app.content',
            test_class=ContentFunctionalTestCase,
            optionflags=(doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)),
    ))



# -*- coding: utf-8 -*-
from Products.Five.browser import BrowserView
from plone.memoize import ram
from zope.component import queryUtility
from zope.i18n.interfaces import ITranslationDomain
import json


def _cache_key(method, self, domain, language):
    return (domain, language,)


class i18njs(BrowserView):

    @ram.cache(_cache_key)
    def _gettext_catalog(self, domain, language):
        td = queryUtility(ITranslationDomain, domain)
        if td is None:
            return
        if language not in td._catalogs:
            baselanguage = language.split('-')[0]
            if baselanguage not in td._catalogs:
                return
            else:
                language = baselanguage
        mo_path = td._catalogs[language][0]
        catalog = td._data[mo_path]._catalog
        if catalog is None:
            td._data[mo_path].reload()
            catalog = td._data[mo_path]._catalog
        return catalog._catalog

    def __call__(self, domain, language=None):
        if domain is None:
            return
        if language is None:
            language = self.request['LANGUAGE']

        catalog = self._gettext_catalog(domain, language)
        response = self.request.response
        response.setHeader('Content-Type', 'application/json; charset=utf-8')
        response.setBody(json.dumps(catalog))
        return response



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

version = '3.3.3'

setup(
    name='plone.app.content',
    version=version,
    description="Content Views for Plone",
    long_description='\n\n'.join([
        open("README.rst").read(),
        open("CHANGES.rst").read(),
    ]),
    classifiers=[
        "Framework :: Plone",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone :: 5.1",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
    ],
    keywords='plone content views viewlet',
    author='Plone Foundation',
    author_email='plone-developers@lists.sourceforge.net',
    url='https://pypi.python.org/pypi/plone.app.content',
    license='GPL version 2',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    zip_safe=False,
    extras_require=dict(
        test=[
            'plone.app.contenttypes',
            'plone.app.testing',
            'mock'
        ]
    ),
    install_requires=[
        'Acquisition',
        'plone.app.widgets',
        'plone.batching',
        'plone.i18n',
        'plone.memoize',
        'plone.protect',
        'Products.CMFCore>=2.2.0dev',
        'Products.CMFDynamicViewFTI',  # required for cmf.ModifyViewTemplate
        'Products.CMFPlone',
        'setuptools',
        'zope.component',
        'zope.container',
        'zope.deferredimport',
        'zope.event',
        'zope.i18n',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.lifecycleevent',
        'zope.publisher',
        'zope.schema',
        'zope.viewlet',
        'Zope2',
    ],
)



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Acquisition import aq_parent
from OFS.CopySupport import CopyError
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from ZODB.POSException import ConflictError
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.widget import ComputedWidgetAttribute
from zExceptions import Unauthorized
from zope import schema
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.container.interfaces import INameChooser
from zope.event import notify
from zope.interface import Interface
from zope.lifecycleevent import ObjectModifiedEvent
import transaction


class LockingBase(BrowserView):

    @property
    def is_locked(self):
        locking_view = queryMultiAdapter(
            (self.context, self.request), name='plone_lock_info')

        return locking_view and locking_view.is_locked_for_current_user()


class DeleteConfirmationForm(form.Form, LockingBase):

    fields = field.Fields()
    template = ViewPageTemplateFile('templates/delete_confirmation.pt')
    enableCSRFProtection = True

    def view_url(self):
        ''' Facade to the homonymous plone_context_state method
        '''
        context_state = getMultiAdapter(
            (self.context, self.request),
            name='plone_context_state'
        )
        return context_state.view_url()

    def more_info(self):
        adapter = queryMultiAdapter(
            (self.context, self.request), name='delete_confirmation_info')
        if adapter:
            return adapter()
        return ""

    @property
    def items_to_delete(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        putils = getToolByName(self.context, 'plone_utils')
        results = catalog({
            'path': '/'.join(self.context.getPhysicalPath()),
            'portal_type': putils.getUserFriendlyTypes(),
        })
        return len(results)

    @button.buttonAndHandler(_(u'Delete'), name='Delete')
    def handle_delete(self, action):
        title = safe_unicode(self.context.Title())
        parent = aq_parent(aq_inner(self.context))

        # has the context object been acquired from a place it should not have
        # been?
        if self.context.aq_chain == self.context.aq_inner.aq_chain:
            parent.manage_delObjects(self.context.getId(), self.request)
            IStatusMessage(self.request).add(
                _(u'${title} has been deleted.', mapping={u'title': title}))
        else:
            IStatusMessage(self.request).add(
                _(u'"${title}" has already been deleted',
                  mapping={u'title': title})
            )

        self.request.response.redirect(parent.absolute_url())

    @button.buttonAndHandler(
        _(u'label_cancel', default=u'Cancel'), name='Cancel')
    def handle_cancel(self, action):
        target = self.view_url()
        return self.request.response.redirect(target)


def valid_id(self):
    # TODO: Do we need an validator here or use the same that's used in
    #       plone.app.dexterity.behaviors.id.IShortName
    return True


class IRenameForm(Interface):
    new_id = schema.ASCIILine(
        title=_(u'label_new_short_name', default=u'New Short Name'),
        description=_(u'help_short_name_url', default=
                      u'Short name is the part that shows up in the URL ' +
                      u'of the item.'),
        constraint=valid_id,
    )

    new_title = schema.TextLine(
        title=_(u'label_new_title', default=u'New Title'),
    )

default_new_id = ComputedWidgetAttribute(
    lambda form: form.context.getId(), field=IRenameForm['new_id'])

default_new_title = ComputedWidgetAttribute(
    lambda form: form.context.Title(), field=IRenameForm['new_title'])


class RenameForm(form.Form):

    fields = field.Fields(IRenameForm)
    template = ViewPageTemplateFile('templates/object_rename.pt')
    enableCSRFProtection = True
    ignoreContext = True

    label = _(u'heading_rename_item', default=u'Rename item')
    description = _(u'description_rename_item', default=
                    u'Each item has a Short Name and a Title, which you can ' +
                    u'change by entering the new details below.')

    def view_url(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name='plone_context_state')
        return context_state.view_url()

    @button.buttonAndHandler(_(u'Rename'), name='Rename')
    def handle_rename(self, action):
        data, errors = self.extractData()
        if errors:
            return

        parent = aq_parent(aq_inner(self.context))
        sm = getSecurityManager()
        if not sm.checkPermission('Copy or Move', parent):
            raise Unauthorized(_(u'Permission denied to rename ${title}.',
                                 mapping={u'title': self.context.title}))

        # Requires cmf.ModifyPortalContent permission
        self.context.title = data['new_title']

        oldid = self.context.getId()
        newid = data['new_id']
        if oldid != newid:
            newid = INameChooser(parent).chooseName(newid, self.context)

            # Requires zope2.CopyOrMove permission

            # manage_renameObjects fires 3 events:
            # 1. ObjectWillBeMovedEvent before anything happens
            # 2. ObjectMovedEvent directly after rename
            # 3. zope.container.contained.notifyContainerModified directly after 2
            # for 2+3 there are subscribers in Products.CMFDynamicViewFTI
            # responsible to change (2) or unset (3) the default_page.

            parent.manage_renameObjects([oldid, ], [str(newid), ])
        else:
            # Object is not reindex if manage_renameObjects is not called
            self.context.reindexObject()

        transaction.savepoint(optimistic=True)
        notify(ObjectModifiedEvent(self.context))

        IStatusMessage(self.request).add(
            _(u"Renamed '${oldid}' to '${newid}'.", mapping={
                u'oldid': oldid, u'newid': newid}))

        self.request.response.redirect(self.view_url())

    @button.buttonAndHandler(_(u'label_cancel', default=u'Cancel'),
                             name='Cancel')
    def handle_cancel(self, action):
        self.request.response.redirect(self.view_url())


class ObjectCutView(LockingBase):

    @property
    def title(self):
        return self.context.Title().decode('utf8')

    @property
    def parent(self):
        return aq_parent(aq_inner(self.context))

    @property
    def canonical_object_url(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name='plone_context_state')
        return context_state.canonical_object_url()

    @property
    def view_url(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name='plone_context_state')
        return context_state.view_url()

    def do_redirect(self, url, message=None, message_type='info',
                    raise_exception=None):
        if message is not None:
            IStatusMessage(self.request).add(message, type=message_type)

        if raise_exception is None:
            return self.request.response.redirect(url)
        raise raise_exception

    def do_action(self):
        if self.is_locked:
            return self.do_redirect(self.view_url,
                                    _(u'${title} is locked and cannot be cut.',
                                        mapping={'title': self.title, }))

        try:
            self.parent.manage_cutObjects(self.context.getId(), self.request)
        except CopyError:
            return self.do_redirect(self.view_url,
                                    _(u'${title} is not moveable.',
                                        mapping={'title': self.title}))

        return self.do_redirect(
            self.view_url,
            _(u'${title} cut.', mapping={'title': self.title}),
            'info'
        )

    def __call__(self):
        authenticator = getMultiAdapter(
            (self.context, self.request), name='authenticator')

        if not authenticator.verify():
            raise Unauthorized

        return self.do_action()


class ObjectCopyView(ObjectCutView):

    def do_action(self):
        try:
            self.parent.manage_copyObjects(self.context.getId(), self.request)
        except CopyError:
            return self.do_redirect(self.view_url,
                                    _(u'${title} is not copyable.',
                                        mapping={'title': self.title}))

        return self.do_redirect(self.view_url,
                                _(u'${title} copied.',
                                    mapping={'title': self.title}))


class ObjectDeleteView(ObjectCutView):

    def do_action(self):
        form = DeleteConfirmationForm(self.context, self.request)
        form.update()

        button = form.buttons['Delete']
        # delete by clicking the form button in delete_confirmation
        form.handlers.getHandler(button)(form, button)


class ObjectPasteView(ObjectCutView):

    def do_action(self):
        if not self.context.cb_dataValid():
            return self.do_redirect(
                self.canonical_object_url,
                _(u'Copy or cut one or more items to paste.'),
                'error'
            )
        try:
            self.context.manage_pasteObjects(self.request['__cp'])
        except ConflictError:
            raise
        except Unauthorized as e:
            self.do_redirect(
                self.canonical_object_url,
                _(u'You are not authorized to paste here.'),
                e
            )
        except CopyError as e:
            error_string = str(e)
            if 'Item Not Found' in error_string:
                return self.do_redirect(
                    self.canonical_object_url,
                    _(u'The item you are trying to paste could not be found. '
                      u'It may have been moved or deleted after you copied or '
                      u'cut it.'),
                    'error',
                )
        except Exception as e:
            if '__cp' in self.request:
                self.do_redirect(self.canonical_object_url, e, 'error', e)

        return self.do_redirect(self.canonical_object_url,
                                _(u'Item(s) pasted.'))



# -*- coding: utf-8 -*-
from OFS.CopySupport import _cb_encode
from OFS.CopySupport import cookie_path
from OFS.Moniker import Moniker
from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.interfaces import IStructureAction
from Products.CMFPlone import PloneMessageFactory as _
from zope.i18n import translate
from zope.interface import implementer


@implementer(IStructureAction)
class CutAction(object):

    order = 1

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_options(self):
        return {
            'title': translate(_('Cut'), context=self.request),
            'id': 'cut',
            'icon': 'scissors',
            'url': self.context.absolute_url() + '/@@fc-cut'
        }


class CutActionView(ContentsBaseAction):
    success_msg = _('Successfully cut items')
    failure_msg = _('Failed to cut items')

    def action(self, obj):
        self.oblist.append(obj)

    def finish(self):
        oblist = []
        for ob in self.oblist:
            if ob.wl_isLocked():
                self.errors.append(_(u'${title} is being edited and cannot be cut.',
                                     mapping={u'title': self.objectTitle(ob)}))
                continue
            if not ob.cb_isMoveable():
                self.errors.append(_(u'${title} is being edited and cannot be cut.',
                                     mapping={u'title': self.objectTitle(ob)}))
                continue
            m = Moniker(ob)
            oblist.append(m.dump())
        cp = (1, oblist)
        cp = _cb_encode(cp)
        resp = self.request.response
        resp.setCookie('__cp', cp, path='%s' % cookie_path(self.request))
        self.request['__cp'] = cp

    def __call__(self):
        self.oblist = []
        return super(CutActionView, self).__call__()
